/************************************************************************

    Dsktool v1.0.0 - LibDsk front end

    Copyright (C) 2005  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "wx/wxprec.h"
#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "dsktool.h"
#include "IdentityForm.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IdentityForm::IdentityForm(wxWindow* parent, const wxString &title, DSK_PDRIVER dsk, DSK_GEOMETRY *dg) :
	wxDialog(parent, -1, title, wxDefaultPosition, wxSize(400, 375))
{
	char buf[40];
   	const char *comp;
	char *comment;
	dsk_err_t err;
	unsigned char drv_status;

	memcpy(&m_dg, dg, sizeof(m_dg));

	comp = dsk_compname(dsk);
	(void)new wxStaticText(this, -1, "Driver:",        wxPoint(5,   5), wxSize(95,  25));
	(void)new wxStaticText(this, -1, dsk_drvdesc(dsk), wxPoint(105, 5), wxSize(300, 25));
	(void)new wxStaticText(this, -1, "Compression:",   wxPoint(5,  25), wxSize(95,  25));
	(void)new wxStaticText(this, -1, comp?comp:"None", wxPoint(105,25), wxSize(300, 25));

	sprintf(buf, "%d", m_dg.dg_sidedness);
	(void)new wxStaticText(this, -1, "Sidedness:",     wxPoint(5,  45), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,45), wxSize(300, 25));
	sprintf(buf, "%d", m_dg.dg_cylinders);
	(void)new wxStaticText(this, -1, "Cylinders:",     wxPoint(5,  65), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,65), wxSize(300, 25));
	sprintf(buf, "%d", m_dg.dg_heads);
	(void)new wxStaticText(this, -1, "Heads:",         wxPoint(5,  85), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,85), wxSize(300, 25));
	sprintf(buf, "%d", m_dg.dg_sectors);
	(void)new wxStaticText(this, -1, "Sectors:",       wxPoint(5, 105), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,105), wxSize(300, 25));
	sprintf(buf, "%d", m_dg.dg_secsize);
	(void)new wxStaticText(this, -1, "Sec. size:",     wxPoint(5, 125), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,125), wxSize(300, 25));
	sprintf(buf, "%d", m_dg.dg_secbase);
	(void)new wxStaticText(this, -1, "1st sector:",    wxPoint(5, 145), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,145), wxSize(300, 25));
	sprintf(buf, "%d", m_dg.dg_datarate);
	(void)new wxStaticText(this, -1, "Data rate:",     wxPoint(5, 165), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,165), wxSize(300, 25));
	(void)new wxStaticText(this, -1, "Record mode:",   wxPoint(5, 185), wxSize(95,  25));
	(void)new wxStaticText(this, -1, m_dg.dg_fm?"FM":"MFM", wxPoint(105,185), wxSize(300, 25));

	sprintf(buf, "0x%02x", m_dg.dg_rwgap);
	(void)new wxStaticText(this, -1, "R/W gap:",       wxPoint(5, 205), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,205), wxSize(300, 25));
	sprintf(buf, "0x%02x", m_dg.dg_fmtgap);
	(void)new wxStaticText(this, -1, "Format gap:",    wxPoint(5, 225), wxSize(95,  25));
	(void)new wxStaticText(this, -1, buf,              wxPoint(105,225), wxSize(300, 25));
	err = dsk_drive_status(dsk, &m_dg, 0, &drv_status);
	(void)new wxStaticText(this, -1, "Drive status:",  wxPoint(5, 245), wxSize(95,  25));
	if (!err)
	{
		sprintf(buf, "0x%02x", drv_status);
		(void)new wxStaticText(this, -1, buf,              wxPoint(105,245), wxSize(300, 25));
	}
	else
	{
		(void)new wxStaticText(this, -1, dsk_strerror(err),wxPoint(105,245), wxSize(300, 25));
	}
	err = dsk_get_comment(dsk, &comment);
	if (err == DSK_ERR_OK && comment != NULL)
	{
		(void)new wxStaticText(this, -1, "Comment:",    wxPoint(5, 265), wxSize(95,  25));
		(void)new wxStaticText(this, -1, buf,           wxPoint(105,265), wxSize(300, 100));
	}
	(void)new wxButton(this, wxID_OK, "OK", wxPoint(160, 305), wxSize(80, 30));
}

IdentityForm::~IdentityForm()
{

}
